/*============================================================================
  File:     EmployeeCaseStudy-AnalyzeStructures.sql

  Summary:  Review the index levels and internal page structures
			of the Employee table case study.
  
  Date:     November 2008
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SYSolutions, Inc.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- These samples use the SQL Server 2008 "IndexInternals" database. 
-- NOTE: This is a compressed SQL Server 2008 backup.

USE IndexInternals
go

-- First review the overall structures
SELECT * FROM sys.dm_db_index_physical_stats
(db_id(), object_id('Employee'), 1, null, 'detailed')
go

SELECT * FROM sys.dm_db_index_physical_stats
(db_id(), object_id('EmployeeHeap'), 3, null, 'detailed')
go
--------------------------------------------------
-- Internals - dissecting the numbers
--------------------------------------------------

-----------------------------
-- First, the CLUSTERED INDEX
-----------------------------
CREATE TABLE #IndexInternals
(
	PageFID			smallint,
	PagePID			int,
	IAMFID			smallint,
	IAMPID			int,
	ObjectID		int,
	IndexID			smallint,
	PartitionNumber	smallint,
	PartitionID		bigint,
	iam_chain_type	varchar(15),
	PageType		tinyint,
	IndexLevel		tinyint,
	NextPageFID		smallint,
	NextPagePID		int,
	PrevPageFID		smallint,
	PrevPagePID		int
)
go

INSERT #IndexInternals (PageFID, PagePID, IAMFID, IAMPID, ObjectID, IndexID, PartitionNumber, PartitionID, iam_chain_type, PageType, IndexLevel, NextPageFID, NextPagePID, PrevPageFID, PrevPagePID)
	EXEC ('DBCC IND(''IndexInternals'', ''Employee'', 1)')
go

SELECT * FROM #IndexInternals
ORDER BY IndexLevel DESC
go

-- reviewing that output, find the root page
-- the root page will have the highest IndexLevel and there will be only one row
-- because of this order by - it should be the first row

dbcc traceon  (3604) 
go

dbcc page ('IndexInternals', 1, 234, 3) -- first page of level 2
	-- 7 rows, this shows the 7 pages (in order) of the intermediate level

-- analyze the intermediate level (which also points to the leaf)
dbcc page ('IndexInternals', 1, 232, 3) -- first page of level 1 (622 rows)
dbcc page ('IndexInternals', 1, 233, 3) -- second page of level 1 (622 rows)
dbcc page ('IndexInternals', 1, 235, 3) -- third page of level 1 (622 rows)
dbcc page ('IndexInternals', 1, 236, 3) -- fourth page of level 1 (622 rows)
dbcc page ('IndexInternals', 1, 237, 3) -- fifth page of level 1 (622 rows)
dbcc page ('IndexInternals', 1, 238, 3) -- sixth page of level 1 (622 rows)
dbcc page ('IndexInternals', 1, 239, 3) -- LAST page of level 1 (268 rows)

-- analyze the leaf level (which IS the data)
dbcc page ('IndexInternals', 1, 168, 3) -- first page of level 0
dbcc page ('IndexInternals', 1, 169, 3) -- second page of level 0
dbcc page ('IndexInternals', 1, 170, 3) -- third page of level 0
dbcc page ('IndexInternals', 1, 2231, 3) -- LAST page of level 0
-- ...
dbcc page ('IndexInternals', 1, 4231, 3) -- LAST page of level 0

	
---------------------------------
-- Second, the NONCLUSTERED INDEX
---------------------------------

TRUNCATE TABLE #IndexInternals
go

INSERT #IndexInternals (PageFID, PagePID, IAMFID, IAMPID, ObjectID, IndexID, PartitionNumber, PartitionID, iam_chain_type, PageType, IndexLevel, NextPageFID, NextPagePID, PrevPageFID, PrevPagePID)
	EXEC ('DBCC IND(''IndexInternals'', ''Employee'', 2)')
go

SELECT * FROM #IndexInternals
ORDER BY IndexLevel DESC
go

dbcc traceon  (3604) 
go

dbcc page ('IndexInternals', 1, 4328, 3) -- first page of level 1
	-- 179 rows, this shows the index pages (in order) of the leaf level 
	-- NOTE: this index doesn't have any intermediate levels.

-- analyze the leaf level (which is a *copy* of the data - in indexed order)
dbcc page ('IndexInternals', 1, 4392, 3) -- first page of level 0
dbcc page ('IndexInternals', 1, 4265, 3) -- second page of level 0
dbcc page ('IndexInternals', 1, 4266, 3) -- third page of level 0
-- ...
dbcc page ('IndexInternals', 1, 4506, 3) -- LAST page of level 0
go

---------------------------------
-- Third, what about a NONCLUSTERED INDEX on a heap
-- in this case the PK (the nc index) has the same
-- order as the data)
---------------------------------

TRUNCATE TABLE #IndexInternals
go

INSERT #IndexInternals (PageFID, PagePID, IAMFID, IAMPID, ObjectID, IndexID, PartitionNumber, PartitionID, iam_chain_type, PageType, IndexLevel, NextPageFID, NextPagePID, PrevPageFID, PrevPagePID)
	EXEC ('DBCC IND(''IndexInternals'', ''EmployeeHeap'', 3)')
go

SELECT * FROM #IndexInternals
ORDER BY IndexLevel DESC
go

dbcc traceon  (3604) 
go

dbcc page ('IndexInternals', 1, 8448, 3) -- first page of level 1
	-- 149 rows, this shows the index pages (in order) of the leaf level 
	-- NOTE: this index doesn't have any intermediate levels.

-- analyze the leaf level (which is a *copy* of the data - in indexed order)
dbcc page ('IndexInternals', 1, 8840, 3) -- first page of level 0
dbcc page ('IndexInternals', 1, 8385, 3) -- second page of level 0
dbcc page ('IndexInternals', 1, 8517, 3) -- third page of level 0
-- ...
dbcc page ('IndexInternals', 1, 8596, 3) -- LAST page of level 0
go

---------------------------------
-- Finally, what about a NONCLUSTERED INDEX on a heap
-- in this case the UK (the nc index) has a *DIFFERENT*
-- order as the data)
---------------------------------

TRUNCATE TABLE #IndexInternals
go

INSERT #IndexInternals (PageFID, PagePID, IAMFID, IAMPID, ObjectID, IndexID, PartitionNumber, PartitionID, iam_chain_type, PageType, IndexLevel, NextPageFID, NextPagePID, PrevPageFID, PrevPagePID)
	EXEC ('DBCC IND(''IndexInternals'', ''EmployeeHeap'', 3)')
go

SELECT * FROM #IndexInternals
ORDER BY IndexLevel DESC
go

dbcc traceon  (3604) 
go

dbcc page ('IndexInternals', 1, 8704, 3) -- first page of level 1
	-- 218 rows, this shows the index pages (in order) of the leaf level 
	-- NOTE: this index doesn't have any intermediate levels.

-- analyze the leaf level (which is a *copy* of the data - in indexed order)
dbcc page ('IndexInternals', 1, 8640, 3) -- first page of level 0
dbcc page ('IndexInternals', 1, 8641, 3) -- second page of level 0
dbcc page ('IndexInternals', 1, 8642, 3) -- third page of level 0
-- ...
dbcc page ('IndexInternals', 1, 8921, 3) -- LAST page of level 0
go